"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.SDOAdapter = void 0;
const Graph_1 = require("./Graph");
const axios_1 = __importDefault(require("axios"));
const utilities_1 = require("./utilities");
const reasoning_1 = require("./reasoning");
class SDOAdapter {
    constructor(paramObj) {
        if (paramObj === null || paramObj === void 0 ? void 0 : paramObj.commit) {
            this.commit = paramObj.commit;
        }
        if (typeof (paramObj === null || paramObj === void 0 ? void 0 : paramObj.onError) === "function") {
            this.onError = paramObj.onError;
        }
        else {
            this.onError = function () {
            };
        }
        if ((paramObj === null || paramObj === void 0 ? void 0 : paramObj.schemaHttps) !== undefined) {
            this.schemaHttps = paramObj.schemaHttps;
        }
        else {
            this.schemaHttps = true;
        }
        if ((paramObj === null || paramObj === void 0 ? void 0 : paramObj.equateVocabularyProtocols) !== undefined) {
            this.equateVocabularyProtocols = paramObj.equateVocabularyProtocols;
        }
        else {
            this.equateVocabularyProtocols = false;
        }
        this.graph = new Graph_1.Graph(this);
    }
    async addVocabularies(vocabArray) {
        vocabArray = (0, utilities_1.toArray)(vocabArray);
        for (const vocab of vocabArray) {
            if ((0, utilities_1.isString)(vocab)) {
                if (vocab.startsWith("www") ||
                    vocab.startsWith("http")) {
                    try {
                        let fetchedVocab = await this.fetchVocabularyFromURL(vocab);
                        if ((0, utilities_1.isString)(fetchedVocab)) {
                            fetchedVocab = JSON.parse(fetchedVocab);
                        }
                        await this.graph.addVocabulary(fetchedVocab, vocab);
                    }
                    catch (e) {
                        throw new Error("The given URL " +
                            vocab +
                            " did not contain a valid JSON-LD vocabulary.");
                    }
                }
                else {
                    try {
                        await this.graph.addVocabulary(JSON.parse(vocab));
                    }
                    catch (e) {
                        throw new Error("Parsing of vocabulary string produced an invalid JSON-LD.");
                    }
                }
            }
            else if ((0, utilities_1.isObject)(vocab)) {
                await this.graph.addVocabulary(vocab);
            }
            else {
                throw new Error("The first argument of the function must be an Array of vocabularies or a single vocabulary (JSON-LD as Object/String)");
            }
        }
        return true;
    }
    async fetchVocabularyFromURL(url) {
        try {
            const res = await axios_1.default.get(url, {
                headers: {
                    Accept: "application/ld+json, application/json",
                },
            });
            return res.data;
        }
        catch (e) {
            throw new Error("Could not find any resource at the given URL.");
        }
    }
    getTerm(id, filter) {
        return this.graph.getTerm(id, filter);
    }
    getAllTerms(filter) {
        const result = [];
        const classesIRIList = this.getListOfClasses(filter);
        const enumerationsIRIList = this.getListOfEnumerations(filter);
        const propertiesIRIList = this.getListOfProperties(filter);
        const dataTypesIRIList = this.getListOfDataTypes(filter);
        const enumerationMembersIRIList = this.getListOfEnumerationMembers(filter);
        for (const c of classesIRIList) {
            result.push(this.getClass(c));
        }
        for (const en of enumerationsIRIList) {
            result.push(this.getEnumeration(en));
        }
        for (const p of propertiesIRIList) {
            result.push(this.getProperty(p));
        }
        for (const dt of dataTypesIRIList) {
            result.push(this.getDataType(dt));
        }
        for (const enm of enumerationMembersIRIList) {
            result.push(this.getEnumerationMember(enm));
        }
        return result;
    }
    getListOfTerms(filter) {
        const result = [];
        result.push(...Object.keys(this.graph.classes));
        result.push(...Object.keys(this.graph.enumerations));
        result.push(...Object.keys(this.graph.properties));
        result.push(...Object.keys(this.graph.dataTypes));
        result.push(...Object.keys(this.graph.enumerationMembers));
        return (0, reasoning_1.applyFilter)({ data: result, filter, graph: this.graph });
    }
    getClass(id, filter) {
        return this.graph.getClass(id, filter);
    }
    getAllClasses(filter) {
        const result = [];
        const classesIRIList = this.getListOfClasses(filter);
        for (const c of classesIRIList) {
            result.push(this.getClass(c));
        }
        return result;
    }
    getListOfClasses(filter) {
        return (0, reasoning_1.applyFilter)({
            data: Object.keys(this.graph.classes),
            filter,
            graph: this.graph,
        });
    }
    getProperty(id, filter) {
        return this.graph.getProperty(id, filter);
    }
    getAllProperties(filter) {
        const result = [];
        const propertiesIRIList = this.getListOfProperties(filter);
        for (const p of propertiesIRIList) {
            result.push(this.getProperty(p));
        }
        return result;
    }
    getListOfProperties(filter) {
        return (0, reasoning_1.applyFilter)({
            data: Object.keys(this.graph.properties),
            filter,
            graph: this.graph,
        });
    }
    getDataType(id, filter) {
        return this.graph.getDataType(id, filter);
    }
    getAllDataTypes(filter) {
        const result = [];
        const dataTypesIRIList = this.getListOfDataTypes(filter);
        for (const dt of dataTypesIRIList) {
            result.push(this.getDataType(dt));
        }
        return result;
    }
    getListOfDataTypes(filter) {
        return (0, reasoning_1.applyFilter)({
            data: Object.keys(this.graph.dataTypes),
            filter,
            graph: this.graph,
        });
    }
    getEnumeration(id, filter) {
        return this.graph.getEnumeration(id, filter);
    }
    getAllEnumerations(filter) {
        const result = [];
        const enumerationsIRIList = this.getListOfEnumerations(filter);
        for (const en of enumerationsIRIList) {
            result.push(this.getEnumeration(en));
        }
        return result;
    }
    getListOfEnumerations(filter) {
        return (0, reasoning_1.applyFilter)({
            data: Object.keys(this.graph.enumerations),
            filter,
            graph: this.graph,
        });
    }
    getEnumerationMember(id, filter) {
        return this.graph.getEnumerationMember(id, filter);
    }
    getAllEnumerationMembers(filter) {
        const result = [];
        const enumerationMembersIRIList = this.getListOfEnumerationMembers(filter);
        for (const enm of enumerationMembersIRIList) {
            result.push(this.getEnumerationMember(enm));
        }
        return result;
    }
    getListOfEnumerationMembers(filter) {
        return (0, reasoning_1.applyFilter)({
            data: Object.keys(this.graph.enumerationMembers),
            filter,
            graph: this.graph,
        });
    }
    getVocabularies(omitStandardVocabs = true) {
        const vocabKeys = Object.keys(this.graph.context);
        const result = {};
        const blacklist = [
            "soa",
            "xsd",
            "rdf",
            "rdfa",
            "rdfs",
            "dcterms",
            "brick",
            "csvw",
            "dc",
            "dcam",
            "dcat",
            "dcmitype",
            "doap",
            "foaf",
            "odrl",
            "org",
            "owl",
            "prof",
            "prov",
            "qb",
            "sh",
            "skos",
            "sosa",
            "ssn",
            "time",
            "vann",
            "void",
            "geo",
            "wgs",
            "ds",
        ];
        vocabKeys.forEach((el) => {
            if ((0, utilities_1.isString)(this.graph.context[el]) &&
                (!omitStandardVocabs || !blacklist.includes(el))) {
                result[el] = this.graph.context[el];
            }
        });
        return result;
    }
    async getLatestSchemaVersion() {
        return await (0, utilities_1.getLatestSchemaVersion)(this.commit);
    }
    async constructURLSchemaVocabulary(version = "latest") {
        return await (0, utilities_1.constructURLSchemaVocabulary)(version, this.schemaHttps, this.commit);
    }
}
exports.SDOAdapter = SDOAdapter;
//# sourceMappingURL=SDOAdapter.js.map